from glob import glob
from logging import ERROR, Formatter
from sys import argv, exit
from argparse import ArgumentParser, RawDescriptionHelpFormatter

from urllib import request

from eis1600.helper.CheckFileEndingActions import CheckFileEndingEIS1600OrEIS1600TMPAction
from p_tqdm import p_uimap
from pandas import read_csv
from tqdm import tqdm

from eis1600.helper.logging import setup_logger
from eis1600.repositories.repo import TEXT_REPO
from eis1600.texts_to_mius.subid_methods import add_ids


def main():
    arg_parser = ArgumentParser(
            prog=argv[0], formatter_class=RawDescriptionHelpFormatter,
            description='''Script to insert UIDs in EIS1600TMP file(s) and thereby converting them to final EIS1600 
            file(s).
-----
Give a single EIS1600 or EIS1600TMP file as input.

Run without input arg to batch process all double-checked and ready files from the OpenITI_EIS1600_Texts directory.
'''
            )
    arg_parser.add_argument('-D', '--debug', action='store_true')
    arg_parser.add_argument(
            'input', type=str, nargs='?',
            help='EIS1600 or EIS1600TMP file to process',
            action=CheckFileEndingEIS1600OrEIS1600TMPAction
            )
    args = arg_parser.parse_args()

    infile = args.input
    debug = args.debug

    if infile:
        add_ids(infile, ids_update=True)
    else:
        print('Download latest version of "_EIS1600 - Text Selection - Serial Source Test - '
              'EIS1600_AutomaticSelectionForReview" from Google Spreadsheets')
        latest_csv = "https://docs.google.com/spreadsheets/d/e/2PACX-1vR60MrlXJjtrd3bid1CR3xK5Pv" \
                    "-aUz1qWEfHfowU1DPsh6RZBvbtW2mA-83drzboIS1fxZdsDO-ny0r/pub?gid=2075964223&single=true&output=csv"
        response = request.urlopen(latest_csv)
        lines = [line.decode('utf-8') for line in response.readlines()]
        csv_path = TEXT_REPO + '_EIS1600 - Text Selection - Serial Source Test - ' \
                               'EIS1600_AutomaticSelectionForReview.csv'
        with open(csv_path, 'w', encoding='utf-8') as csv_fh:
            csv_fh.writelines(lines)

        print('Saved as csv in ' + TEXT_REPO)

        df = read_csv(csv_path, usecols=['Book Title', 'PREPARED']).dropna()
        df_processable_files = df.loc[df['PREPARED'].str.fullmatch('double-checked|ready')]

        infiles = []

        print('URIs of files for whom no .EIS1600 file was found')
        for uri in df_processable_files['Book Title']:
            author, text = uri.split('.')
            text_path = TEXT_REPO + 'data/' + author + '/' + uri + '/'
            text_file = glob(text_path + '*.EIS1600')
            if text_file:
                infiles.append(text_file[0])
            else:
                print(uri)

        if not infiles:
            print(
                    'There are no more EIS1600 files to process'
            )
            exit()

        print('\nAdd IDs')
        formatter = Formatter('%(message)s\n\n\n')
        logger = setup_logger('sub_ids', TEXT_REPO + 'sub_ids.log', ERROR, formatter)
        res = []
        count = 0
        if debug:
            x = 1
            for i, infile in tqdm(list(enumerate(infiles[x:]))):
                print(i + x, infile)
                try:
                    add_ids(infile)
                except ValueError as e:
                    logger.error(f'{infile}\n{e}\n\n')
                    count += 1

            print(f'{len(infiles)-count}/{len(infiles)} processed')
        else:
            res += p_uimap(add_ids, infiles)

    print('Done')






from logging import Formatter, INFO
from glob import glob

from eis1600.helper.logging import setup_logger
from eis1600.repositories.repo import TEXT_REPO, get_ready_and_double_checked_texts
from eis1600.texts_to_mius.check_formatting_methods import check_formatting
from eis1600.texts_to_mius.subid_methods import add_ids


def main():
    """Script to check file formatting based on text selection."""

    df_ready, df_double_checked = get_ready_and_double_checked_texts()
    double_checked_files = []
    ready_files = []

    missing_texts = []
    for uri in df_double_checked['Book Title']:
        author, text = uri.split('.')
        text_path = TEXT_REPO + 'data/' + author + '/' + uri + '/'
        text_file = glob(text_path + '*.EIS1600')
        if text_file:
            double_checked_files.append(text_file[0])
        else:
            missing_texts.append(uri)

    if missing_texts:
        print('URIs for double-checked files for whom no .EIS1600 file was found')
        for uri in missing_texts:
            print(uri)
        print('\n')

    missing_texts = []
    for uri in df_ready['Book Title']:
        author, text = uri.split('.')
        text_path = TEXT_REPO + 'data/' + author + '/' + uri + '/'
        tmp_file = glob(text_path + '*.EIS1600TMP')
        eis_file = glob(text_path + '*.EIS1600')
        if tmp_file and not eis_file:
            ready_files.append(tmp_file[0])
        elif tmp_file and eis_file:
            double_checked_files.append(eis_file[0])
            # print(f'{uri} (both TMP and EIS1600)')
        elif eis_file and not tmp_file:
            double_checked_files.append(eis_file[0])
            missing_texts.append(f'{uri} (no TMP but EIS1600)')
        else:
            missing_texts.append(f'{uri} (missing)')

    if missing_texts:
        print('URIs for ready files for whom no .EIS1600TMP file was found')
        for uri in missing_texts:
            print(uri)
        print('\n')

    formatter = Formatter('%(message)s\n\n\n')
    logger = setup_logger('mal_formatted_texts', TEXT_REPO + 'mal_formatted_texts.log', INFO, formatter)

    logger.info('insert_uids')
    print('Insert UIDs into ready texts')

    x = 0
    for i, file in enumerate(ready_files[x:]):
        print(i + x, file)
        try:
            add_ids(file)
        except ValueError as e:
            logger.error(f'{file}\n{e}')

    logger.info('\n\n\ndisassemble_text')
    print('Check formatting for double-checked and ready texts')

    texts = double_checked_files + [r.replace('TMP', '') for r in ready_files]

    count = 0
    x = 0
    for i, text in enumerate(texts[x:]):
        print(i + x, text)
        try:
            check_formatting(text)
        except ValueError as e:
            count += 1
            logger.error(e)
        except FileNotFoundError:
            print(f'Missing: {text}')

    print(f'\n{count}/{len(texts)} texts need fixing\n')

    print('Done')




from sys import argv
from argparse import ArgumentParser, RawDescriptionHelpFormatter

from eis1600.helper.CheckFileEndingActions import CheckFileEndingEIS1600OrEIS1600TMPAction
from eis1600.texts_to_mius.check_formatting_methods import check_formatting


def main():
    arg_parser = ArgumentParser(
            prog=argv[0], formatter_class=RawDescriptionHelpFormatter,
            description=''
    )
    arg_parser.add_argument(
            'input', type=str, nargs='?',
            help='EIS1600 or EIS1600TMP file to process',
            action=CheckFileEndingEIS1600OrEIS1600TMPAction
    )

    args = arg_parser.parse_args()

    infile = args.input

    check_formatting(infile)
